import { world, system } from "@minecraft/server"

export function cooldown() {

    // Weapon Cooldown Manager
    const weaponCooldowns = {
        "bey:ravager_horn": { maxCooldown: 3600, displayName: "Ravager Horn" },
        "bey:mjolnir": { maxCooldown: 2400, displayName: "Mjolnir" },
        "bey:void_staff_shulker": { maxCooldown: 1200, displayName: "Void Staff" },
        "bey:ender_blade": { maxCooldown: 900, displayName: "Dragon Katana" },
        "bey:litch_staff": { maxCooldown: 900, displayName: "Litch Staff" },
        "bey:shadow_blade": { maxCooldown: 1800, displayName: "Shadow Blade" },
        "bey:sonic_crossbow_charged": { maxCooldown: 900, displayName: "Sonic Crossbow" },
        "bey:crimson_chainsword": { maxCooldown: 1800, displayName: "Crimson ChainSword" },
        "bey:excalibur": { maxCooldown: 2400, displayName: "Excalibur" },
        "bey:golem_hammer": { maxCooldown: 1800, displayName: "Golem Hammer" }
    };

    // Function to increment all cooldowns
    function incrementCooldowns(player) {
        for (const weaponId in weaponCooldowns) {
            const cooldownKey = `cooldown_${weaponId}`;
            const currentCooldown = player.getDynamicProperty(cooldownKey) ?? 0;

            //PreAdder
            if (player.getDynamicProperty(cooldownKey) == undefined) {
                player.setDynamicProperty(cooldownKey, 0);
            }

            // Increment the cooldown if it's less than max
            if (currentCooldown > 0) {
                player.setDynamicProperty(cooldownKey, currentCooldown + 1);
            }
            if (currentCooldown >= weaponCooldowns[weaponId].maxCooldown) {
                player.setDynamicProperty(cooldownKey, 0);
            }
        }
    }

    // Function to handle cooldown display
    function handleCooldownDisplay(player, weaponId) {
        const weaponData = weaponCooldowns[weaponId];
        if (!weaponData) return;

        const { maxCooldown, displayName } = weaponData;
        const cooldownKey = `cooldown_${weaponId}`;
        const currentCooldown = player.getDynamicProperty(cooldownKey) ?? 0;

        // Only display the cooldown if it's active
        if (currentCooldown > 0 && currentCooldown < maxCooldown) {
            const secLeft = Math.ceil((maxCooldown - currentCooldown) / 20);
            const progressRatio = secLeft / (maxCooldown / 20);
            const totalBars = 3;
            const filledBars = Math.round(progressRatio * totalBars);
            const progressBar = `§a${'■'.repeat(filledBars)}§7${'■'.repeat(totalBars - filledBars)}`;

            player.onScreenDisplay.setActionBar(
                `§6 §e${displayName} §cCooldown §6[§b${secLeft} sec§6] ${progressBar}`
            );
        }
    }

    system.runInterval(() => {
        world.getAllPlayers().forEach((player) => {

            //Anti Multi Crimson ChainSword
            let playerHeld = player.getComponent(`equippable`).getEquipment(`Mainhand`);
            const crimsonCooldown = player.getDynamicProperty("cooldown_bey:crimson_chainsword");
            if (playerHeld && playerHeld.typeId === "bey:crimson_chainsword_cursed" && crimsonCooldown != 0) {
                player.runCommand(`/replaceitem entity @s slot.weapon.mainhand 0 bey:crimson_chainsword`);
            }

            //Item Lore
            const inv = player.getComponent("inventory").container;
            let slot = inv.size;
            while (slot--) {
                const itemSlot = inv.getSlot(slot);
                switch (true) {
                    case (!itemSlot.hasItem()): continue;
                    case (itemSlot.typeId == "bey:golem_hammer"):
                        itemSlot.setLore(["", "§r§6A hammer of immense weight.", "§r§6Forged from ancient stone.",
                            "----------", "§r§l§6Power : §d* * *", "----------",
                            "§r§lAbility:", "§r§dInteract to dash forward with a leap", "§r§dSlam the ground when you land", "§r§dKnocking foes away in a 6 block radius"]);
                        break;
                    case (itemSlot.typeId == "bey:excalibur"):
                        itemSlot.setLore(["", "§r§6The legendary sword of kings.", "§r§6It shines with divine power",
                            "----------", "§r§l§6Power : §d* * *", "----------",
                            "§r§lAbility:", "§r§dInteract to get invinsibility for 3 hits"]);
                        break;
                    case (itemSlot.typeId == "bey:crimson_chainsword" || itemSlot.typeId == "bey:crimson_chainsword_cursed"):
                        itemSlot.setLore(["", "§r§6A blade stained with blood", "§r§6Its roar echoes in battle.",
                            "----------", "§r§l§6Power : §d* * * *", "----------",
                            "§r§lAbility:", "§r§dInteract to curse your blade", "§r§dCursed blade will wither your foe", "§r§dthe first time you hit it.", "§r§dKilling will give you strength", "§r§dbuff and unCurse your blade", "§r§d+1 Attack Damage When Cursed"]);
                        break;
                    case (itemSlot.typeId == "bey:ravager_horn"):
                        itemSlot.setLore(["", "§r§6An ancient horn whose call across the land", "§r§6summons fear into enemies.",
                            "----------", "§r§l§6Power : §d* * *", "----------",
                            "§r§lAbility:", "§r§dInteract to summon a rideable ravager", "§r§dRavager:", "§r§dHits And Knocks All Entity Nearby Except You"]);
                        break;
                    case (itemSlot.typeId == "bey:mjolnir"):
                        itemSlot.setLore(["", "§r§6A hammer forged by the gods,", "§r§6crackling with the fury of", "§r§6endless storms and lightning.",
                            "----------", "§r§l§6Power : §d* * *", "----------",
                            "§r§lAbility:", "§r§dInteract to throw the hammer", "§r§dhitten block or entity", "§r§dWill summon lightning in that location", "§r§dYou will get the hammer back in 2 sec"]);
                        break;
                    case (itemSlot.typeId == "bey:artemis_bow"):
                        itemSlot.setLore(["", "§r§6A bow blessed by the goddess of the hunt,", "§r§6rumored to have a divine origin.",
                            "----------", "§r§l§6Power : §d* * ", "----------",
                            "§r§lAbility:", "§r§dShotten arrow have a 1/3 chance of", "§r§dstriking hurt entity with lightning"]);
                        break;
                    case (itemSlot.typeId == "bey:emerald_blade"):
                        itemSlot.setLore(["§r§6Forged in the heart of a lush emerald forest,", "§r§6it hums with the energy of nature.",
                            "----------", "§r§l§6Power : §d* * * *", "----------",
                            "§r§lAbility:", "§r§dif u have a emerald block and hit a entity", "§r§dit will use the emerald block", "§r§dand do pierce damage through all armor"]);
                        break;
                    case (itemSlot.typeId == "bey:ender_blade"):
                        itemSlot.setLore(["§r§6A mysterious blade,", "§r§6said to be woven from the fabric of the End.",
                            "----------", "§r§l§6Power : §d*", "----------",
                            "§r§lAbility:", "§r§dThrow ender pearl by interacting", "§r§dHold to get speed boost."]);
                        break;
                    case (itemSlot.typeId == "bey:ender_scythe"):
                        itemSlot.setLore(["§r§6A reaper's tool born from the void,", "§r§6whispering the secrets of the End.",
                            "----------", "§r§l§6Power : §d* *", "----------",
                            "§r§lAbility:", "§r§dGet health boost every 5 hit"]);
                        break;
                    case (itemSlot.typeId == "bey:litch_staff"):
                        itemSlot.setLore(["§r§6A arcane staff wielded by forgotten sorcerers,", "§r§6pulsing with necromantic power.",
                            "----------", "§r§l§6Power : §d* * *", "----------",
                            "§r§lAbility:", "§r§dShoot a sonic waves by interacting,", "§r§dslowing all entities trapped in it"]);
                        break;
                    case (itemSlot.typeId == "bey:magma_club"):
                        itemSlot.setLore(["§r§6A weapon crafted from molten rock,", "§r§6burning with the fury of a volcano.",
                            "----------", "§r§l§6Power : §d* * *", "----------",
                            "§r§lAbility:", "§r§dHold in Main/Offhand to get fire resistance,", "§r§dhit a entity to make it burn"]);
                        break;
                    case (itemSlot.typeId == "bey:midas_sword"):
                        itemSlot.setLore(["§r§6A golden blade imbued with the", "§r§6legendary touch of King Midas.",
                            "----------", "§r§l§6Power : §d* * * *", "----------",
                            "§r§lAbility:", "§r§dKill non-player entity to gain strength buff,", "§r§dkill a player to summon a golden statue", "§r§dof them, interact with the statue", "§r§dto gain a health buff"]);
                        break;
                    case (itemSlot.typeId == "bey:shadow_blade"):
                        itemSlot.setLore(["§r§6A weapon cloaked in darkness,", "§r§6feeding on the fears of its wielder.",
                            "----------", "§r§l§6Power : §d* * *", "----------",
                            "§r§lAbility:", "§r§dInteract to go invisible", "§r§d[Hitting will remove your effect],", "§r§dhit a entity to wither them"]);
                        break;
                    case (itemSlot.typeId == "bey:sonic_crossbow" || itemSlot.typeId == "bey:sonic_crossbow_charged"):
                        itemSlot.setLore(["", "§r§6Engineered to channel supersonic force,", "§r§6this crossbow cracks the air with every shot.",
                            "----------", "§r§l§6Power : §d* * *", "----------",
                            "§r§lAbility:", "§r§dLoad the crossbow and shoot to shoot", "§r§da sonic boom instead of arrows"]);
                        break;
                    case (itemSlot.typeId == "bey:void_staff_ender"):
                        itemSlot.setLore(["§r§6An artifact of the End,", "§r§6drawing its power from the endless void.",
                            "----------", "§r§l§6Power : §d* * *", "----------",
                            "§r§lAbility:", "§r§dInteract to summon teleport portal 1,", "§r§dwhile in its radius interact again", "§r§dto summon teleport portal 2", "§r§dgo to teleport portal 1 to", "§r§dteleport to the other portal", "§r§d[Sneak And Interact To Switch Modes]"]);
                        break;
                    case (itemSlot.typeId == "bey:void_staff_shulker"):
                        itemSlot.setLore(["§r§6An artifact of the End,", "§r§6drawing its power from the endless void.",
                            "----------", "§r§l§6Power : §d* * *", "----------",
                            "§r§lAbility:", "§r§dInteract to shoot a shulker bullet that,", "§r§dgives the entity levitation that it hits", "§r§d[Sneak And Interact To Switch Modes]"]);
                        break;
                    case (itemSlot.typeId == "bey:war_pickaxe"):
                        itemSlot.setLore(["§r§6Once a miner's tool,", "§r§6now reforged into a weapon of conquest.",
                            "----------", "§r§l§6Power : §d* * *", "----------",
                            "§r§lAbility:", "§r§dHit a entity to knock them away,", "§r§dMines in a 3x3x3 Area"]);
                        break;
                }
            }

            // Cooldown
            incrementCooldowns(player);
            if (playerHeld) {
                const weaponId = playerHeld.typeId;
                handleCooldownDisplay(player, weaponId);
            }
        })
    })
}
