import { world, ItemComponentTypes, system, EntityComponentTypes } from "@minecraft/server"

export function entityHitEntity() {

    let ENDER_SCYTHE_HIT = new Map();

    system.runInterval(() => {
        world.getAllPlayers().forEach(player => {
            let playerName = player.name;

            //Player Hold Effects
            let playerHeld = player.getComponent(`equippable`).getEquipment(`Mainhand`);
            let playerHeldOffhand = player.getComponent(`equippable`).getEquipment(`Offhand`);

            //Golem Hammer
            if (player.hasTag(`golemLifted`) && player.isOnGround || player.hasTag(`golemLifted`) && player.isInWater) {
                player.dimension.spawnParticle(`bey:golem_hammer_dirt`, player.location)
                player.runCommand(`/scriptevent beyond:knockback`);
                player.playSound(`random.anvil_land`, player.location);
                player.removeTag(`golemLifted`)
            }

            //Magma Club Hold Effects
            if (playerHeld) {
                if (playerHeld.typeId == "bey:magma_club") {
                    player.addEffect("fire_resistance", 20, { amplifier: 1 });
                }
            }
            if (playerHeld) {
                if (playerHeld.typeId == "bey:ender_blade") {
                    player.addEffect("speed", 20, { amplifier: 1 });
                }
            }
            if (playerHeldOffhand) {
                if (playerHeldOffhand.typeId == "bey:magma_club") {
                    player.addEffect("fire_resistance", 20, { amplifier: 1 });
                }
            }


            //Ender Scythe Hit Counter
            if (!ENDER_SCYTHE_HIT.has(playerName)) {
                ENDER_SCYTHE_HIT.set(playerName, 0);
            }

            //Ravager Horn
            player.dimension.getEntities().forEach(entity => {

                if (entity.typeId == "bey:ridden_ravager") {
                    let currentHealth = entity.getComponent(`health`).currentValue;
                    entity.nameTag = `${player.name}: §l§8Ravager Left = §4${Math.floor(currentHealth)}§7/§445 Hearts`
                }


                if (entity.typeId == "bey:ridden_ravager") {
                    let entityLoc = entity.getViewDirection();
                    entity.dimension.getEntities({ location: entity.location, maxDistance: 2 }).forEach(entity2 => {
                        if (entity2.nameTag != entity.nameTag.split(":")[0]
                            && entity2.typeId !== "bey:ridden_ravager") {
                            try {
                                entity2.applyDamage(5);
                                entity2.applyKnockback(entityLoc.x, entityLoc.y, 1, 0.7)
                            }
                            catch { }
                        }
                    })
                }
            })
        })
    }, 1)

    world.afterEvents.projectileHitBlock.subscribe((event) => {
        let projectile = event.projectile;
        let block = event.getBlockHit().block;

        if (projectile.typeId == "bey:mjolnir_entity") {
            block.dimension.spawnEntity(`lightning_bolt`, block.location)
            projectile.runCommand("/tp @s ~ ~100 ~")
            system.runTimeout(() => {
                projectile.runCommand("/kill @s")
            })
        }

        if (projectile.typeId == "bey:custom_shulker_bullet") {
            projectile.runCommand("/tp @s ~ ~100 ~")
            system.runTimeout(() => {
                projectile.runCommand("/kill @s")
            })
        }
    })

    world.afterEvents.projectileHitEntity.subscribe((event) => {
        let projectile = event.projectile;
        let hurtEntity = event.getEntityHit().entity;

        if (projectile.typeId == "bey:mjolnir_entity") {
            hurtEntity.dimension.spawnEntity(`lightning_bolt`, hurtEntity.location)
        }

        if (projectile.typeId == "bey:custom_shulker_bullet") {
            hurtEntity.addEffect("levitation", 40, { amplifier: 2 })
        }

        if (projectile.typeId == "bey:custom_artemis_arrow") {
            let x = Math.floor(Math.random() * 3);
            if (x == 2) {
                hurtEntity.dimension.spawnEntity("lightning_bolt", hurtEntity.location)
            }
        }
    })

    world.afterEvents.entityDie.subscribe((event) => {
        let player = event.damageSource.damagingEntity;
        let playerHeld = player.getComponent(`equippable`).getEquipment(`Mainhand`);

        let killedEntity = event.deadEntity;
        if (playerHeld && playerHeld.typeId == "bey:crimson_chainsword_cursed") {
            player.addEffect("strength", 80, { amplifier: 1 });
            player.runCommand(`/replaceitem entity @s slot.weapon.mainhand 0 bey:crimson_chainsword`);
            player.setDynamicProperty("cooldown_bey:crimson_chainsword", 1)

        }
        if (playerHeld && playerHeld.typeId == "bey:midas_sword" && killedEntity.typeId != "minecraft:player") {
            player.addEffect("strength", 20, { amplifier: 1 });
        }
        if (playerHeld && playerHeld.typeId == "bey:midas_sword" && killedEntity.typeId == "minecraft:player") {
            player.dimension.spawnEntity("bey:golden_player_statue", killedEntity.location)
        }
    })

    world.afterEvents.entityHurt.subscribe((event) => {
        let hurtEntity = event.hurtEntity;

        //Excalibur
        const excaliCooldown = hurtEntity.getDynamicProperty("cooldown_bey:excalibur");
        if (excaliCooldown && excaliCooldown == -3) {
            hurtEntity.setDynamicProperty("cooldown_bey:excalibur", -2)
        }
        if (excaliCooldown && excaliCooldown == -2) {
            hurtEntity.setDynamicProperty("cooldown_bey:excalibur", -1)
        }
        if (excaliCooldown && excaliCooldown == -1) {
            hurtEntity.runCommand(`/effect @s resistance 0 0`)
            hurtEntity.setDynamicProperty("cooldown_bey:excalibur", 1)
        }
    })

    world.afterEvents.entityHitEntity.subscribe((event) => {
        let player = event.damagingEntity;
        let playerView = player.getViewDirection();
        let playerHeld = player.getComponent(`equippable`).getEquipment(`Mainhand`);
        let currentHit = ENDER_SCYTHE_HIT.get(player.nameTag) || 0;


        let hurtEntity = event.hitEntity;

        //Excalibur
        const excaliCooldown = player.getDynamicProperty("cooldown_bey:excalibur");
        if (excaliCooldown == -3) {
            player.setDynamicProperty("cooldown_bey:excalibur", -2)
        }
        if (excaliCooldown == -2) {
            player.setDynamicProperty("cooldown_bey:excalibur", -1)
        }
        if (excaliCooldown == -1) {
            player.runCommand(`/effect @s resistance 0 0`)
            player.setDynamicProperty("cooldown_bey:excalibur", 1)
        }

        //Crimson ChainSword Hit
        if (playerHeld && playerHeld.typeId == "bey:crimson_chainsword_cursed") {
            if (!hurtEntity.hasTag(`CrimsonCursed`)) {
                hurtEntity.addEffect(`wither`, 70, { amplifier: 3 })
            }
            hurtEntity.addTag(`CrimsonCursed`);
        }

        //Void Staff Hit Kill
        if (playerHeld && playerHeld.typeId == "bey:void_staff_ender") {
            if (["bey:void_staff_teleporter_1", "bey:void_staff_teleporter_2"].includes(hurtEntity.typeId)) {
                hurtEntity.runCommandAsync("/tp @s ~ ~100 ~");
                system.runTimeout(() => {
                    hurtEntity.runCommandAsync("/kill @s");
                }, 10)
            }
        }

        //EnderWand Hit
        if (playerHeld && playerHeld.typeId == "bey:ender_wand") {
            if (hurtEntity.isSprinting) {
                hurtEntity.runCommandAsync(`/ damage @s 4 entity_attack entity ${player.nameTag} `);
            }
        }

        // Emerald Blade
        if (playerHeld && playerHeld.typeId == "bey:emerald_blade") {
            let totalEmeraldBlocks = 0;

            const inv = player.getComponent("inventory").container;
            let slot = inv.size;

            while (slot--) {
                const itemSlot = inv.getSlot(slot);
                if (!itemSlot.hasItem()) continue;

                if (itemSlot.typeId == "minecraft:emerald_block") {
                    totalEmeraldBlocks += itemSlot.amount;
                }
            }

            if (totalEmeraldBlocks > 6) {
                let damage = Math.min(Math.floor(totalEmeraldBlocks / 6) * 2, 15);
                hurtEntity.runCommand(`/damage @s ${damage} entity_attack entity ${player.nameTag}`)
            }

            // Perform the emerald blade action if there are emerald blocks
            if (totalEmeraldBlocks > 6) {
                hurtEntity.runCommandAsync("/particle bey:emerald_blade_hit");
                hurtEntity.applyDamage(4, { damagingEntity: player, cause: "freezing" });
            }
        }


        //Magma Club Hits
        if (playerHeld && playerHeld.typeId == "bey:magma_club") {
            hurtEntity.setOnFire(6);
        }

        //Ender Scythe Hits
        if (playerHeld && playerHeld.typeId == "bey:ender_scythe") {
            ENDER_SCYTHE_HIT.set(player.nameTag, currentHit + 1);
            if (currentHit > 2) {
                player.addEffect("regeneration", 50, { amplifier: 3 });
                ENDER_SCYTHE_HIT.set(player.nameTag, 0);
            }
        }

        //Shadow Blade Withered Hit
        if (playerHeld && playerHeld.typeId == "bey:shadow_blade") {
            hurtEntity.addEffect("wither", 20, { amplifier: 2 })
        }

        //Shadow Blade Invisibility Remover
        if (playerHeld && player.hasTag("shadowBladeInv")) {
            player.runCommandAsync("/effect @s invisibility 0 0 true")
            player.removeTag("shadowBladeInv");
        }

        //War Pickaxe Knockback
        if (playerHeld && playerHeld.typeId == "bey:war_pickaxe") {

            hurtEntity.applyKnockback(playerView.x, playerView.y, 1.8, 0.5)
            for (let i = 0; i < 3; i++) {
                hurtEntity.dimension.spawnParticle("minecraft:camera_shoot_explosion", hurtEntity.location)
            }
        }
    })

    system.afterEvents.scriptEventReceive.subscribe(({ id, sourceEntity: player }) => {
        if (!player || id !== 'beyond:knockback') return;

        const entities = player.dimension.getEntities({ location: player.location, maxDistance: 6 });

        for (const entity of entities) {
            if (entity.id === player.id) continue;
            const { x, z } = Vector.subtract(entity.location, player.location);
            entity?.applyKnockback(x, z, 3, 0.8);
        }
    });

    class Vector {
        static multiply(vectorA, value) {
            return {
                x: vectorA.x * (typeof value === 'number' ? value : value.x),
                y: vectorA.y * (typeof value === 'number' ? value : value.y),
                z: vectorA.z * (typeof value === 'number' ? value : value.z)
            };
        }
        static add(vectorA, vectorB) {
            return {
                x: vectorA.x + (vectorB.x ?? 0),
                y: vectorA.y + (vectorB.y ?? 0),
                z: vectorA.z + (vectorB.z ?? 0)
            };
        }
        static subtract(vectorA, vectorB) {
            return {
                x: vectorA.x - (vectorB.x ?? 0),
                y: vectorA.y - (vectorB.y ?? 0),
                z: vectorA.z - (vectorB.z ?? 0)
            };
        }
        static distance(vectorA, vectorB) {
            return Math.sqrt(Math.pow(vectorA.x - vectorB.x, 2)
                + Math.pow(vectorA.y - vectorB.y, 2)
                + Math.pow(vectorA.z - vectorB.z, 2));
        }
    }
}