import { world, system, EntityComponentTypes, ItemComponentTypes } from "@minecraft/server"

export function weaponInteract() {

    function shootProjectile(entity, typeId, power) {
        const headLoc = entity.getHeadLocation();
        const viewVector = entity.getViewDirection();
        const direction = { x: headLoc.x + viewVector.x, y: headLoc.y + viewVector.y, z: headLoc.z + viewVector.z };
        const vel = { x: viewVector.x * power, y: viewVector.y * power, z: viewVector.z * power };
        const projectile = entity.dimension.spawnEntity(typeId, direction).getComponent(EntityComponentTypes.Projectile);
        projectile.owner = entity;
        projectile.shoot(vel);
    };

    if (!world.scoreboard.getObjective("entityTimerSS")) { world.scoreboard.addObjective("entityTimerSS", "entityTimerSS"); }

    system.runInterval(() => {
        world.getAllPlayers().forEach(player => {

            player.dimension.getEntities().forEach(entity => {

                if (entity.typeId == "bey:mjolnir_entity") {
                    system.runTimeout(() => {
                        try {
                            entity.runCommand(`/kill @s`)
                        }
                        catch { }
                    }, 24)
                }

                if (entity.typeId == "bey:golden_player_statue") {
                    let { x, y, z } = entity.location;

                    try {
                        let currentTimer = world.scoreboard.getObjective("entityTimerSS").getScore(entity);
                        if (currentTimer > 0) {
                            entity.runCommandAsync("/scoreboard players add @s entityTimerSS 1");
                            entity.teleport({ x, y: y + 100, z });
                        }
                        if (currentTimer == 10) {
                            entity.runCommandAsync("/kill @s");
                        }
                    } catch { }
                }
            });
        });
    }, 1);

    world.afterEvents.itemStartUse.subscribe((event) => {
        let player = event.source;
        let itemStack = event.itemStack;

        if (itemStack.typeId == "bey:sonic_crossbow" || itemStack.typeId == "bey:artemis_bow") {
            player.playSound("crossbow.loading.start", { location: player.location })
        }
    })

    world.afterEvents.itemCompleteUse.subscribe((event) => {
        let player = event.source;
        let itemStack = event.itemStack;

        //Sonic Crossbow Loading
        if (itemStack.typeId == "bey:sonic_crossbow") {

            //Has Arrow
            const inv = player.getComponent("inventory").container;
            let slot = inv.size;
            while (slot--) {
                const itemSlot = inv.getSlot(slot);
                if (!itemSlot.hasItem()) continue;
                if (itemSlot.typeId == "minecraft:arrow") {
                    player.playSound("note.bell", { volume: 10, pitch: 0.421, location: player.location })
                    player.runCommand("/replaceitem entity @s slot.weapon.mainhand 0 bey:sonic_crossbow_charged")
                    player.runCommand("/clear @s minecraft:arrow 0 1")
                    break;
                }
                else { }
            }
        }
        //Sonic Crossbow Explosion [Cooldown Done]
        const sonicCooldown = player.getDynamicProperty("cooldown_bey:sonic_crossbow_charged");
        if (itemStack.typeId == "bey:sonic_crossbow_charged" && sonicCooldown == 0) {

            player.setDynamicProperty("cooldown_bey:sonic_crossbow_charged", 1)

            const replaceableBlocks = [
                "grass_block",
                "stone",
                "andesite",
                "granite",
                "diorite",
                "gravel"
            ];

            for (let i = 1; i <= 18; i++) {
                player.playSound("mob.warden.attack", { volume: 10, pitch: 0.9, location: player.location });
                player.runCommand(`execute at @s positioned ^ ^1.5 ^${i * 2} run particle minecraft:sonic_explosion ~~~`);
                player.runCommandAsync("/replaceitem entity @s slot.weapon.mainhand 0 bey:sonic_crossbow");

                system.runTimeout(() => {
                    player.getEntitiesFromViewDirection({ maxDistance: 28 }).forEach((entity) => {
                        let viewDir = entity.entity.getViewDirection();
                        entity.entity.applyKnockback(viewDir.x, viewDir.y, 0.6, 0.6);

                        entity.entity.runCommand(`/damage @s 15 entity_attack entity ${player.nameTag}`)

                        // 7x7 area replacement
                        for (let x = -5; x <= 5; x++) {
                            for (let z = -5; z <= 5; z++) {
                                if (Math.random() < 0.06) {
                                    replaceableBlocks.forEach(blockType => {
                                        entity.entity.runCommand(
                                            `fill ~${x} ~-4 ~${z} ~${x} ~3 ~${z} minecraft:sculk replace minecraft:${blockType}`
                                        );
                                        if (Math.random() < 0.005) {
                                            entity.entity.runCommand(
                                                `fill ~${x} ~-2 ~${z} ~${x} ~ ~${z} minecraft:sculk_sensor replace air`
                                            );
                                        }
                                        if (Math.random() < 0.005) {
                                            entity.entity.runCommand(
                                                `fill ~${x} ~-2 ~${z} ~${x} ~ ~${z} minecraft:sculk_catalyst replace air`
                                            );
                                        }
                                        if (Math.random() < 0.002) {
                                            entity.entity.runCommand(
                                                `fill ~${x} ~-2 ~${z} ~${x} ~ ~${z} minecraft:sculk_shrieker replace air`
                                            );
                                        }
                                    });
                                }
                            }
                        }
                    });
                });
            }
        }
    })

    world.afterEvents.itemUse.subscribe((event) => {
        let player = event.source;
        let item = event.itemStack;
        let { x, y, z } = player.location;
        let playerView = player.getViewDirection();

        //Golem Hammer
        const golemCooldown = player.getDynamicProperty("cooldown_bey:golem_hammer");
        if (item.typeId === "bey:golem_hammer" && golemCooldown == 0) {
            player.applyKnockback(playerView.x, playerView.z, 2, 1);
            player.setDynamicProperty("cooldown_bey:golem_hammer", 1)
            player.addTag(`golemLifted`)
        }

        //Excalibur
        const excaliCooldown = player.getDynamicProperty("cooldown_bey:excalibur");
        if (item.typeId === "bey:excalibur" && excaliCooldown == 0) {
            player.addEffect(`resistance`, 99999, { amplifier: 255 });
            player.setDynamicProperty("cooldown_bey:excalibur", -3)
        }

        //Crimson Chainsword
        const crimsonCooldown = player.getDynamicProperty("cooldown_bey:crimson_chainsword");
        if (item.typeId === "bey:crimson_chainsword" && crimsonCooldown == 0) {
            player.runCommand(`/replaceitem entity @s slot.weapon.mainhand 0 bey:crimson_chainsword_cursed`);
        }

        // Ravager Horn [Cooldown Done]
        const ravagerHornCooldown = player.getDynamicProperty("cooldown_bey:ravager_horn");
        if (item.typeId === "bey:ravager_horn" && ravagerHornCooldown == 0) {

            player.dimension.getEntities().forEach(entity => {
                if (entity.typeId == "bey:ridden_ravager") {
                    let entityTagPrefix = entity.nameTag.split(":")[0];
                    if (entityTagPrefix == player.name) {
                        entity.runCommand(`/kill @s`)
                    }
                }
            })

            player.runCommand(`/summon bey:ridden_ravager "${player.name}:Ravager"`);
            player.playSound(`raid.horn`, player.location)
            player.setDynamicProperty("cooldown_bey:ravager_horn", 1)

            const durability = item.getComponent(ItemComponentTypes.Durability);

            if (durability.damage === durability.maxDurability) {
                player.playSound('random.break');
                player.getComponent('equippable').setEquipment('Mainhand', undefined);
            } else {
                durability.damage++;
                player.getComponent('equippable').setEquipment('Mainhand', item);
            }

        }


        //Mjolnir  [Cooldown Done]
        const mjolnirCooldown = player.getDynamicProperty("cooldown_bey:mjolnir");
        if (item.typeId == "bey:mjolnir" && mjolnirCooldown == 0) {
            player.setDynamicProperty("cooldown_bey:mjolnir", 1)

            player.runCommandAsync(`/clear @s bey:mjolnir 0 1`)
            shootProjectile(player, "bey:mjolnir_entity", 1.9);
            player.playSound(`random.anvil_break`, player.location)
            system.runTimeout(() => {
                player.runCommand(`/give @s bey:mjolnir 1`)
            }, 45)
        }

        //Shulker Staff [Cooldown Done]
        const shulkerCooldown = player.getDynamicProperty("cooldown_bey:void_staff_shulker");
        if (item.typeId == "bey:void_staff_shulker" && shulkerCooldown == 0 && !player.isSneaking) {
            player.setDynamicProperty("cooldown_bey:void_staff_shulker", 1)
            shootProjectile(player, "bey:custom_shulker_bullet", 2);
        }
        if (item.typeId == "bey:void_staff_shulker" && player.isSneaking) {
            player.runCommand("/replaceitem entity @s slot.weapon.mainhand 0 bey:void_staff_ender")
        }

        //Void Staff
        if (item.typeId == "bey:void_staff_ender") {
            if (!player.isSneaking) {
                if (player.hasTag("inTp1Range")) {
                    player.runCommandAsync(`/tp @e[type=bey:void_staff_teleporter_2,name="${player.name}"] ~ ~100 ~`)
                    system.runTimeout(() => {
                        player.runCommandAsync(`/kill @e[type=bey:void_staff_teleporter_2,name="${player.name}"]`)
                        player.runCommandAsync(`/summon bey:void_staff_teleporter_2 "${player.name}"`)
                    }, 5)
                }
                if (!player.hasTag("inTp1Range")) {
                    player.runCommandAsync(`/tp @e[type=bey:void_staff_teleporter_1,name="${player.name}"] ~ ~100 ~`)
                    player.runCommandAsync(`/tp @e[type=bey:void_staff_teleporter_2,name="${player.name}"] ~ ~100 ~`)
                    system.runTimeout(() => {
                        player.runCommandAsync(`/kill @e[type=bey:void_staff_teleporter_1,name="${player.name}"]`)
                        player.runCommandAsync(`/kill @e[type=bey:void_staff_teleporter_2,name="${player.name}"]`)
                        player.runCommandAsync(`/summon bey:void_staff_teleporter_1 "${player.name}"`)
                    }, 5)
                }
            }
            if (player.isSneaking) {
                player.runCommand("/replaceitem entity @s slot.weapon.mainhand 0 bey:void_staff_shulker")
            }
        }

        //Dragon Katana Teleport [Cooldown Done]
        const katanaCooldown = player.getDynamicProperty("cooldown_bey:ender_blade");
        if (item.typeId == "bey:ender_blade" && katanaCooldown == 0) {
            player.setDynamicProperty("cooldown_bey:ender_blade", 1)
            shootProjectile(player, "bey:custom_ender_pearl", 1.2);
        }


        //Litch Staff Slow Radius [Cooldown Done]
        const litchCooldown = player.getDynamicProperty("cooldown_bey:litch_staff");
        if (item.typeId == "bey:litch_staff" && litchCooldown == 0) {
            player.setDynamicProperty("cooldown_bey:litch_staff", 1)
            for (let i = 1; i <= 9; i++) {
                player.runCommand(`execute at @s positioned ^ ^1.5 ^${i * 2} run particle bey:litch_staff_explosion ~~~`);
            }
            system.runTimeout(() => {
                player.getEntitiesFromViewDirection({ maxDistance: 18 }).forEach((entity) => {
                    let viewDir = entity.entity.getViewDirection()
                    if (entity.entity.nameTag != `${player.name}`) {
                        entity.entity.applyKnockback(viewDir.x, viewDir.y, 1, 1);
                        entity.entity.addEffect(`slowness`, 100, { amplifier: 1 })
                    }
                })
            })
        }

        //Shadow Blade Invisibility [Cooldown Down]
        const shadowCooldown = player.getDynamicProperty("cooldown_bey:shadow_blade");
        if (item.typeId == "bey:shadow_blade" && shadowCooldown == 0) {
            player.setDynamicProperty("cooldown_bey:shadow_blade", 1)

            for (let i = 0; i < 3; i++) {
                player.dimension.spawnParticle("minecraft:campfire_smoke_particle",
                    { x, y: y + i, z: z + (i / 2) });
            }
            player.addEffect("invisibility", 120, { amplifier: 1 });
            player.addEffect("speed", 120, { amplifier: 1, showParticles: false });
            player.playSound("beacon.activate", { location: player.location })
            player.addTag("shadowBladeInv");
        }
    })
}